# -*- Mode: cperl -*-
#--------------------------------------------------------------------
# Copyright (C) 2000, 2001 by MandrakeSoft.
# Chmouel Boudjnah <chmouel@mandrakesoft.com>.
#
# Redistribution of this file is permitted under the terms of the GNU 
# Public License (GPL)
#--------------------------------------------------------------------
# $Id: common.pm,v 1.21 2001/10/16 11:51:33 chmouel Exp $
#--------------------------------------------------------------------

package common;
use strict;

my $debug = 0;

my $sysconfig = $debug ? "./installkernel.sysconfig" : "/etc/sysconfig/installkernel";

my $mdksub = "smp|enterprise|secure";

sub check_default_entry {
    # dont ask me why i need to do that :p
    my $foo = $_[0];
    my %entry = %$foo;
    my $foo1 = $_[1];
    my %main = %$foo1;
    
    # check if the default entry if there is an initrd pointing to the
    # right kernel.
    while (my $act = each %entry) {
	
	# find the default ?
	if ($act =~ /^$main{default}$/) {
	    # there is an initrd ?
	    if ($entry{$act}{initrd}) {
		
		my $initrd_version_of_entry;
		my $kernel_version_of_entry;	    
		
		#too much code :p but i see problems coming and more
		#different code between the two case

		my $initrd_of_entry = $entry{$act}{initrd};
		$initrd_of_entry = readlink ($initrd_of_entry) if -l $initrd_of_entry;

#ifdef DOCHECK_FOR_NON_MDK_KERNEL
# 		#Not mdk kernel in that case we just get the name basically
# 		if ($initrd_of_entry !~ /mdk/ and $initrd_of_entry =~ m,(/boot/)?initrd-(.*)\.img?, ) {
# 		    $initrd_version_of_entry = $2 if $2;
# 		}
#endif		
		#A mdk kernel so make sure to get only the mdk one
		if ($initrd_of_entry =~ /mdk/ and $initrd_of_entry =~ m,(/boot/)?initrd-(.*)mdk($mdksub|.*)?\.img?, ) {
		    my $s = $3;
		    $initrd_version_of_entry = $2 . "mdk" if $2;
		    #<chmou dark-satanique mode>
		    #	That tricky since if it is a mdk kernel it should
		    #	match $mdksub so since here if it's don't
		    #	match we don't add to the
		    #	$initrd_version_of_entry which mean wrong name
		    #	and fail.
		    #</chmou dark-satanique mode>
		    $initrd_version_of_entry .= $s if $s =~ /$mdksub/;
		}

		my $kernel_of_entry = $entry{$act}{kernel};
		$kernel_of_entry = readlink ($kernel_of_entry) if -l $kernel_of_entry;

#ifdef DOCHECK_FOR_NON_MDK_KERNEL
# 		#Not mdk kernel in that case we just get the name basically
# 		if ($kernel_of_entry !~ /mdk/ and $kernel_of_entry =~ m,(/boot/)?vmlinuz-(.*)$, ) {
# 		    $kernel_version_of_entry = $2 if $2;
# 		}
#endif		
		#A mdk kernel so make sure to get only the mdk one
		if ($kernel_of_entry =~ /mdk/ and $kernel_of_entry =~ m,(/boot/)?vmlinuz-(.*)mdk($mdksub|.*)?, ) {
		    my $s = $3;
		    $kernel_version_of_entry = $2 . "mdk" if $2;
		    if ($s and $s !~ /$mdksub/) {
			print "Looks like you have an unknown mdk kernel, please contact Chmouel <chmouel\@mandrakesoft.com>\n";
			# We still continue and don't make it fail
			# fail, since we want to add a new kernel
			# anyway
			return;
		    }
		    $kernel_version_of_entry .= $s if $s;
		}
		
		print "kernel=$kernel_version_of_entry, initrd=$initrd_version_of_entry\n" if $debug;
		if ($kernel_version_of_entry !~ /^$initrd_version_of_entry$/) {
		    print STDERR ("look like there was a problem, the default vmlinuz version is not the same \nof the initrd\n");
		    exit(1);
		}
	    }
	}
     }
}   

sub get_first_boot {
    my ($type, $resolved_link) = @_;
    my $boot = '/boot/';
    my @list;
    opendir D, $boot;
    for (readdir(D)) { 
	if (/vmlinuz-.*$/i and not /$resolved_link/ and not /vmlinuz-($mdksub)/) {
	    push @list, $_ 
	}
    }
    close D;
    @list = reverse(sort(@list));
    ( my $first_kernel = shift(@list) ) =~ s|vmlinuz-||;
    system("perl /usr/share/loader/make-initrd $first_kernel") unless $debug;
    return $first_kernel;
}

sub do_check {
    my ($label, $version, $remove) = @_;
    if ($remove && !$label){
	print("not removing entry, entry doen't exists\n");
	exit;
    }	
    if ($label && !$remove) {
	print("not adding entry, $label already exists\n");
	exit;
    }
    unless (-f "/boot/vmlinuz-$version") {
	print("look like i cannot find the specified kernel /boot/vmlinuz-$version\n") ;
	exit;
    }
}

sub symlinkf { unlink $_[1]; symlink $_[0], $_[1] }
sub cat_ { local *F; open F, $_[0] or $_[1] ? die "cat of file $_[0] failed: $!\n" : return; my @l = <F>; wantarray ? @l : join '', @l }

sub getoptions {
    my $lilo = shift;
    my ($options, $foo);
    my %var = getVarsFromSh("$sysconfig");
    my $cmdline = grep(/NOCMDLINE/, $var{OPTIONS}) ? undef : cat_("/proc/cmdline");chomp $cmdline;
    foreach $foo (split(/\s+/, join(" ", $var{OPTIONS}, $cmdline)))  {
	next if $foo =~ /auto/ and $lilo;
	next if $foo =~ /root=/;
	next if $foo =~ /^r[ow]/;
	next if $foo =~ /BOOT_IMAGE=/;
	next if $foo =~ /NOCMDLINE/;
	next if (grep /$foo/, $options);
	$options .= "$foo ";
    }
    return $options;
}

sub getroot_from_sysconfig {
    my %var = getVarsFromSh("$sysconfig");
    return (grep /root=\/\S+/, split(" ", $var{OPTIONS}))[0];
}
    
sub getroot {
    my $root;
    $root = getroot_from_sysconfig();
    $root = (grep /root=\/\S+/, split(" ", cat_('/proc/cmdline')))[0] unless $root;
    $root = getroot_from_fstab() unless $root;
    ($root = `/usr/sbin/rdev`) =~ s|(\S+)\s+.*$|root=$1| unless $root;chomp $root;
    warn("There is no root= defined you are going in trouble\n") unless $root;
    return $root;
}

sub getroot_from_fstab {
    my $part;
    local *F;
    open F, '/etc/fstab';
    while (<F>) {
	my @s = split ' ';
	$part = $s[0] if $s[1] =~ m|/$| and not $part;
    }
    close F;
    return "root=" . $part;
}	
   

sub getVarsFromSh(%) {
    my %l;
    local *F; open F, $_[0] or return;
    local $_;
    while (<F>) {
	s/#.*//; # remove comments
	my ($v, $val, $val2) =
	  /^\s*			# leading space
	   (\w+) =		# variable
	   (
   	       "([^"]*)"	# double-quoted text
   	     | '([^']*)'	# single-quoted text
   	     | [^'"\s]+		# normal text
           )
           \s*$			# end of line
          /x or next;
	$l{$v} = defined $val2 ? $val2 : $val;
    }
    %l;
}

sub get_kernel_type {
    $_ = shift;
    return $1 if (/.*($mdksub)$/);
    return undef;
}

1;#wonderful perl
